VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "DB_Markup"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

Private data_ As Excel.Worksheet
Private sources_ As Excel.Worksheet

Public Function Init(iData As Excel.Worksheet, iSources As Excel.Worksheet)
  Set data_ = iData
  Set sources_ = iSources
End Function

Public Function RemoveAll()
  Call data_.UsedRange.Offset(1, 0).ClearContents
  Call sources_.UsedRange.Offset(1, 0).ClearContents
End Function

Public Function GetLinkFor(nDataRow&) As ItemLink
  If data_.Cells(nDataRow, S_DATA_ID) = vbNullString Then _
    Exit Function
  Dim sPath$: sPath = FindSourcePath(data_.Cells(nDataRow, S_DATA_SOURCE))
  If sPath = vbNullString Then _
    Exit Function
  
  Dim iLink As New ItemLink
  iLink.start_ = data_.Cells(nDataRow, S_DATA_START)
  iLink.finish_ = data_.Cells(nDataRow, S_DATA_FINISH)
  iLink.path_ = sPath
  
  Set GetLinkFor = iLink
End Function

Public Function ImportFromExcel(iSource As Excel.Worksheet) As Boolean
  ImportFromExcel = False
  
  Dim iPath As New API_Path: Call iPath.FromString(iSource.Cells(FIRST_ROW, S_IN_SOURCE))
  Dim intSource As ItemSource: Set intSource = PrepareSourceInput(iPath)
  If intSource Is Nothing Then _
    Exit Function
  
  Dim rowOut&: rowOut = intSource.rowOut_
  Dim rowIn&: rowIn = FIRST_ROW
  Do While iSource.Cells(rowIn, S_IN_START) <> vbNullString
    data_.Cells(rowOut, S_DATA_ID) = intSource.sourceID_ & "-" & VBA.Format(rowIn - 1, "0000")
    data_.Cells(rowOut, S_DATA_SOURCE) = intSource.sourceID_
    data_.Cells(rowOut, S_DATA_START) = iSource.Cells(rowIn, S_IN_START)
    data_.Cells(rowOut, S_DATA_FINISH) = iSource.Cells(rowIn, S_IN_FINISH)
    data_.Cells(rowOut, S_DATA_TYPE) = iSource.Cells(rowIn, S_IN_TYPE)
    data_.Cells(rowOut, S_DATA_TEXT) = iSource.Cells(rowIn, S_IN_TEXT)
    data_.Cells(rowOut, S_DATA_COMMENT) = iSource.Cells(rowIn, S_IN_COMMENT)
    rowOut = rowOut + 1
    rowIn = rowIn + 1
  Loop
  
  ImportFromExcel = True
End Function

Public Function ImportFromWord(iSource As InfoDocument) As Boolean
  ImportFromWord = False
  
  Dim frags As Collection: Set frags = iSource.GetBasicFragmentsCompound()
  Dim iPath As New API_Path: Call iPath.FromString(iSource.doc_.FullName).GlobalToServer
  Dim intSource As ItemSource: Set intSource = PrepareSourceInput(iPath)
  If intSource Is Nothing Then _
    Exit Function
    
  Dim rowOut&: rowOut = intSource.rowOut_
  Dim rowStart&: rowStart = intSource.rowOut_
  Dim nItem&: nItem = 1
  Dim aFrag As ItemFragment
  For Each aFrag In frags
    Dim theRange As Word.Range: Set theRange = iSource.doc_.Range(aFrag.start_, aFrag.end_)
    
    data_.Cells(rowOut, S_DATA_ID) = intSource.sourceID_ & "-" & VBA.Format(nItem - 1, "0000")
    data_.Cells(rowOut, S_DATA_SOURCE) = intSource.sourceID_
    data_.Cells(rowOut, S_DATA_START) = aFrag.start_
    data_.Cells(rowOut, S_DATA_FINISH) = aFrag.end_
    data_.Cells(rowOut, S_DATA_TYPE) = aFrag.id_
    data_.Cells(rowOut, S_DATA_TEXT) = theRange.Text
    data_.Cells(rowOut, S_DATA_COMMENT) = ExtractCommentText(theRange)
    
    nItem = nItem + 1
    rowOut = rowOut + 1
  Next aFrag
  
  Call data_.Range(data_.Cells(rowStart, S_DATA_ID), data_.Cells(rowOut - 1, S_DATA_COMMENT)).Sort( _
    Key1:=data_.Cells(rowStart, S_DATA_START), _
    Order1:=xlAscending, _
    Key2:=data_.Cells(rowStart, S_DATA_FINISH), _
    Order2:=xlDescending, _
    Header:=xlNo)
  
  ImportFromWord = True
End Function

' ======
Private Function PrepareSourceInput(iPath As API_Path) As ItemSource
  Dim iSource As New ItemSource
  Dim nRow&: nRow = FIRST_ROW
  Dim sPath$: sPath = iPath.Text
  Do While sources_.Cells(nRow, S_SRC_ID) <> vbNullString
    If sources_.Cells(nRow, S_SRC_PATH) = sPath Then
      If Not UserInteraction.AskQuestion(QM_CLEAR_BEFORE_READD) Then _
        Exit Function
      Call RemoveSource(nRow)
    Else
      nRow = nRow + 1
    End If
  Loop
  
  iSource.rowSource_ = nRow
  iSource.sourceID_ = Excel.Application.WorksheetFunction.Max(sources_.Range("A:A")) + 1
  iSource.rowOut_ = GetLastDataRow
  
  Call OutputSource(iSource, iPath)
  
  Set PrepareSourceInput = iSource
End Function

Private Function OutputSource(iSource As ItemSource, iPath As API_Path)
  sources_.Cells(iSource.rowSource_, S_SRC_ID) = iSource.sourceID_
  sources_.Cells(iSource.rowSource_, S_SRC_NAME) = iPath.BaseName
  Dim iCell As Excel.Range: Set iCell = sources_.Cells(iSource.rowSource_, S_SRC_PATH)
  iCell = iPath.Text
  Call XLUpdateHyperlink(iCell, iPath.Text)
End Function

Private Function RemoveSource(nRow&)
  Dim nSource&: nSource = sources_.Cells(nRow, S_SRC_ID)
  Call sources_.Rows(nRow).Delete

  Dim nDataRow&: nDataRow = FIRST_ROW
  Do While data_.Cells(nDataRow, S_DATA_ID) <> vbNullString
    If data_.Cells(nDataRow, S_DATA_SOURCE) = nSource Then
      data_.Rows(nDataRow).Delete
    Else
      nDataRow = nDataRow + 1
    End If
  Loop
End Function

Private Function GetLastDataRow() As Long
  Dim nRow&: nRow = FIRST_ROW
  Do While data_.Cells(nRow, S_DATA_ID) <> vbNullString
    nRow = nRow + 1
  Loop
  GetLastDataRow = nRow
End Function

Private Function FindSourcePath(nSource&) As String
  Dim nRow&: nRow = FIRST_ROW
  Do While sources_.Cells(nRow, S_SRC_ID) <> vbNullString
    If sources_.Cells(nRow, S_SRC_ID) = nSource Then
      FindSourcePath = sources_.Cells(nRow, S_SRC_PATH)
      Exit Function
    End If
    nRow = nRow + 1
  Loop
End Function
