VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "InfoConfig"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
'================ -     =========================
Option Explicit

Public sourcePath_ As String
Public hash_ As Long

Public categories_ As Scripting.Dictionary ' map of (catID, InfoCategory)

Private nextID_ As Long

Private Sub Class_Initialize()
  Set categories_ = New Scripting.Dictionary
  nextID_ = 1
End Sub

Public Property Get IsLoaded() As Boolean
  IsLoaded = sourcePath_ <> ""
End Property

Public Function Load(sFileName$) As Boolean
  Load = False
  
  Dim fso As New Scripting.FileSystemObject
  If Not fso.FileExists(sFileName) Then _
    Exit Function
  
  Dim wrapper As New API_WordWrapper
  Call wrapper.CreateApplication(bIsVisible:=False)
  Dim theDoc As Word.Document: Set theDoc = wrapper.OpenDocument(sFileName, bReadOnly:=True, bTrackMRU:=False)
  If theDoc Is Nothing Then _
    Exit Function
  If theDoc.Tables.Count = 0 Then
    Call wrapper.ReleaseDocument
    Exit Function
  End If
  
  Load = ScanCategories(theDoc.Tables(1))
  If Load Then
    sourcePath_ = sFileName
    Call UpdateHash
  End If
  
  Call wrapper.ReleaseDocument
  
  Load = True
End Function

Public Function Contains(catID&) As Boolean
  Contains = categories_.Exists(catID)
End Function

Public Function GetAt(catID&) As InfoCategory
  If Contains(catID) Then _
    Set GetAt = categories_(catID)
End Function

Public Function CountIncomplete() As Long
  Dim iCat As InfoCategory
  Dim catID As Variant
  For Each catID In categories_
    Set iCat = categories_(catID)
    If Not iCat.IsComplete Then _
      CountIncomplete = CountIncomplete + 1
  Next catID
End Function

Public Function CountCombos() As Long
  Dim iCat As InfoCategory
  Dim catID As Variant
  For Each catID In categories_
    Set iCat = categories_(catID)
    If iCat.IsCombo Then _
      CountCombos = CountCombos + 1
  Next catID
End Function

' Throws: ERR_CAT_NAME_TAKEN
Public Function AddBasicCategory(sName$, Optional sInfo$ = "") As InfoCategory
  If sName = vbNullString Then _
    Exit Function
  If Not FindName(sName) Is Nothing Then _
    Call Err.Raise(ERR_CAT_NAME_TAKEN, Source:=sName)
    
  Dim newCat As New InfoCategory: Call newCat.InitBasic(nextID_, sName, sInfo)
  nextID_ = nextID_ + 1
  Call categories_.Add(newCat.id_, newCat)
  Call UpdateHash
  Set AddBasicCategory = newCat
End Function

' Throws: ERR_CAT_NAME_TAKEN, ERR_COMBO_TAKEN, ERR_UNKNOWN_CAT_ID
Public Function AddCombinedCategory(sName$, iArguments As Scripting.Dictionary, Optional sInfo$ = "") As InfoCategory
  Dim existingCombo As InfoCategory: Set existingCombo = FindComposition(iArguments)
  If Not existingCombo Is Nothing Then _
    Call Err.Raise(ERR_COMBO_TAKEN, existingCombo.id_)
  
  Dim iComposition As Scripting.Dictionary: Set iComposition = ReduceComposition(iArguments)
  If iComposition Is Nothing Then _
    Call Err.Raise(ERR_UNKNOWN_CAT_ID)
    
  Dim iNewCat As InfoCategory: Set iNewCat = AddBasicCategory(sName, sInfo)
  If iNewCat Is Nothing Then _
    Exit Function
    
  Set iNewCat.composition_ = iComposition
  
  Call UpdateHash
  Set AddCombinedCategory = iNewCat
End Function

Public Function EraseCategory(targetID&) As Boolean
  EraseCategory = InternalErase(targetID)
  If Not EraseCategory Then _
    Exit Function
    
  Dim toDelete As New Collection
  Dim iCat As InfoCategory
  Dim catID As Variant
  For Each catID In categories_
    Set iCat = categories_(catID)
    If iCat.composition_.Exists(targetID) Then _
      Call toDelete.Add(catID)
  Next catID
  
  For Each catID In toDelete
    Call InternalErase(CStr(catID))
  Next catID
  Call UpdateHash
End Function

Public Function ReduceComposition(iCats As Scripting.Dictionary) As Scripting.Dictionary
  Dim iComp As New Scripting.Dictionary
  Dim catID As Variant
  For Each catID In iCats
    Dim iCat As InfoCategory: Set iCat = GetAt(CLng(catID))
    If iCat Is Nothing Then _
      Exit Function
    If iCat.IsCombo Then
      Dim catElement As Variant
      For Each catElement In iCat.composition_
        iComp(catElement) = 0
      Next catElement
    Else
       iComp.Item(catID) = 0
    End If
  Next catID
  Set ReduceComposition = iComp
End Function

Public Function GetChildrenOf(targetID&) As Scripting.Dictionary
  Dim iParent As InfoCategory: Set iParent = GetAt(targetID)
  If iParent Is Nothing Then _
    Exit Function
  
  Dim iChildren As New Scripting.Dictionary
  Dim catID As Variant
  Dim iCat As InfoCategory
  For Each catID In categories_
    If catID <> targetID Then
      Set iCat = categories_(catID)
      If iParent.IsIncludedIn(iCat) Then _
        Call iChildren.Add(catID, 0)
    End If
  Next catID
  Set GetChildrenOf = iChildren
End Function

' Throws: ERR_STYLE_TAKEN, ERR_FORMAT_TAKEN, ERR_STYLE_NO_FORMAT
Public Function AssignStyle(targetID&, wStyle As Word.Style, iDoc As Word.Document) As Boolean
  AssignStyle = False
  Dim target As InfoCategory: Set target = GetAt(targetID)
  If target Is Nothing Then _
    Exit Function
  If target.style_ = wStyle.NameLocal Then _
    Exit Function
  If Not FindStyle(wStyle.NameLocal) Is Nothing Then _
    Call Err.Raise(ERR_STYLE_TAKEN)

  Dim iFormat As New InfoFormat: Call iFormat.InitFromWordDiff(iDoc.Styles(wdStyleNormal), wStyle)
  If iFormat.IsEmpty Then _
    Call Err.Raise(ERR_STYLE_NO_FORMAT, Source:=wStyle.NameLocal)
  
  Dim iClone As InfoCategory: Set iClone = FindFormat(iFormat, iDoc)
  If Not iClone Is Nothing Then _
    If iClone.id_ <> targetID Then _
      Call Err.Raise(ERR_FORMAT_TAKEN, Source:=iClone.name_)

  If iDoc.FullName <> sourcePath_ Then _
    Call WordCopyStyle(iDoc.Application, iDoc.FullName, sourcePath_, wStyle.NameLocal)
  target.style_ = wStyle.NameLocal
  
  Call UpdateHash
  AssignStyle = True
End Function

' Throws ERR_INCOMPLETE_COMPOSITION, ERR_CANNOT_ADD_STYLE
Public Function CreateStyle(targetID&, sStyleName$, iDoc As Word.Document) As Boolean
  CreateStyle = False
  Dim target As InfoCategory: Set target = GetAt(targetID)
  If target Is Nothing Then _
    Exit Function
  If target.IsComplete Or Not target.IsCombo Then _
    Exit Function
  
  Dim iCombo As New Collection
  Dim parentID As Variant
  Dim iParent As InfoCategory
  For Each parentID In target.composition_
    Set iParent = GetAt(CLng(parentID))
    If Not iParent.IsComplete Then _
      Call Err.Raise(ERR_INCOMPLETE_COMPOSITION)
    Call iCombo.Add(iDoc.Styles(iParent.style_))
  Next parentID
  
  Dim newStyle As Word.Style: Set newStyle = CreateStyleCombination(sStyleName, iCombo, iDoc)
  If newStyle Is Nothing Then _
    Call Err.Raise(ERR_CANNOT_ADD_STYLE)
  
  If iDoc.FullName <> sourcePath_ Then _
    Call WordCopyStyle(iDoc.Application, iDoc.FullName, sourcePath_, newStyle.NameLocal)
  target.style_ = newStyle.NameLocal
  
  Call UpdateHash
  CreateStyle = True
End Function

Public Function FindStyle(sStyle$) As InfoCategory
  Dim iCat As InfoCategory
  Dim catID As Variant
  For Each catID In categories_
    Set iCat = categories_(catID)
    If iCat.style_ = sStyle Then
      Set FindStyle = iCat
      Exit Function
    End If
  Next catID
End Function

Public Function FindName(sName$) As InfoCategory
  Dim iCat As InfoCategory
  Dim catID As Variant
  For Each catID In categories_
    Set iCat = categories_(catID)
    If iCat.name_ = sName Then
      Set FindName = iCat
      Exit Function
    End If
  Next catID
End Function

Public Function FindComposition(iComposition As Scripting.Dictionary) As InfoCategory
  If iComposition.Count = 0 Then _
    Exit Function
    
  Dim iBasicComp As Scripting.Dictionary: Set iBasicComp = ReduceComposition(iComposition)
  If iBasicComp Is Nothing Then _
    Exit Function
    
  Dim iCat As InfoCategory
  Dim catID As Variant
  For Each catID In categories_
    Set iCat = categories_(catID)
    If CompareDeep(iBasicComp, iCat.composition_) = 0 Then
      Set FindComposition = iCat
      Exit Function
    End If
  Next catID
End Function

Public Function FindFormat(iTarget As InfoFormat, iDoc As Word.Document) As InfoCategory
  Dim catID As Variant
  For Each catID In categories_
    Dim iCat As InfoCategory: Set iCat = categories_(catID)
    Dim iFormat As InfoFormat: Set iFormat = iCat.GetFormat(iDoc)
    If Not iFormat Is Nothing Then
      If iTarget.Compare(iFormat) = 0 Then
        Set FindFormat = iCat
        Exit Function
      End If
    End If
  Next catID
End Function

Public Function SyncDocument(target As Word.Document)
  If hash_ <> GetMetadata(target, VAR_HASH) Or sourcePath_ <> GetMetadata(target, VAR_MUPATH) Then
    Call SyncStyles(target.Application, target.FullName)
    Call SetMetadata(target, VAR_HASH, hash_)
    Call SetMetadata(target, VAR_MUPATH, sourcePath_)
  End If
End Function

Public Function SyncStyles(wordApp As Word.Application, sDestination$)
  Dim catID As Variant
  For Each catID In categories_
    Dim iCat As InfoCategory: Set iCat = categories_(catID)
    If iCat.IsComplete Then _
      Call WordCopyStyle(wordApp, sourcePath_, sDestination, iCat.style_)
  Next catID
End Function

Public Function SaveAs(outPath$) As Boolean
  SaveAs = False
  
  Dim wrapper As New API_WordWrapper
  Dim outDoc As Word.Document: Set outDoc = wrapper.OpenDocument(outPath, bReadOnly:=False)
  If outDoc Is Nothing Then _
    Exit Function
  
  Call SyncStyles(wrapper.Application, outPath)
  sourcePath_ = outPath
  
  Call InternalSaveTo(outDoc)
  Call wrapper.ReleaseDocument
  
  SaveAs = True
End Function

Public Function Save() As Boolean
  Save = False
  
  Dim wrapper As New API_WordWrapper
  Dim outDoc As Word.Document: Set outDoc = wrapper.OpenDocument(sourcePath_, bReadOnly:=False)
  If outDoc Is Nothing Then _
    Exit Function
    
  Call InternalSaveTo(outDoc)
  Call wrapper.ReleaseDocument
  
  Save = True
End Function

' =========
Private Function InternalErase(targetID&) As Boolean
  InternalErase = categories_.Exists(targetID)
  If InternalErase Then _
    Call categories_.Remove(targetID)
End Function

Private Function ScanCategories(iSource As Word.Table)
  ScanCategories = False
  
  Call categories_.RemoveAll
  
  Dim nRow&
  Dim iCat As InfoCategory
  For nRow = WORD_FIRST_ROW To iSource.Rows.Count Step 1
    Dim sID$: sID = TrimWhitespace(iSource.Cell(nRow, WORD_CAT_ID).Range.Text)
    If Not IsNumeric(sID) Then _
      Exit Function

    Set iCat = New InfoCategory
    With iCat
      .id_ = VBA.CLng(sID)
      .name_ = TrimWhitespace(iSource.Cell(nRow, WORD_CAT_NAME).Range.Text)
      .info_ = TrimWhitespace(iSource.Cell(nRow, WORD_CAT_INFO).Range.Text)
      .style_ = TrimWhitespace(iSource.Cell(nRow, WORD_CAT_STYLE).Range.Text)
    End With
    Dim sComp$: sComp = TrimWhitespace(iSource.Cell(nRow, WORD_CAT_COMP).Range.Text)
    If Not iCat.ImportComposition(sComp) Then _
      Exit Function
    
    Call categories_.Add(iCat.id_, iCat)
    
    If nextID_ <= iCat.id_ Then _
      nextID_ = iCat.id_ + 1
  Next nRow
  
  ScanCategories = True
End Function

Private Function InternalSaveTo(outDoc As Word.Document)
  Call PrepareOutput(outDoc)
  Call OutputCategories(outDoc.Tables(1))
  Call outDoc.Save
End Function

Private Function PrepareOutput(target As Word.Document)
  Dim catTable As Word.Table
  If target.Tables.Count = 0 Then
    target.Sections(1).PageSetup.Orientation = wdOrientLandscape
    Call target.Range.InsertParagraphAfter
    Set catTable = target.Tables.Add(target.Paragraphs.Last.Range, 1, 5)
    With catTable
      .Cell(1, WORD_CAT_ID).Range = "ID"
      .Cell(1, WORD_CAT_NAME).Range = ""
      .Cell(1, WORD_CAT_STYLE).Range = ""
      .Cell(1, WORD_CAT_INFO).Range = ""
      .Cell(1, WORD_CAT_COMP).Range = ""
      
      .Borders.InsideLineStyle = wdLineStyleSingle
      .Borders.OutsideLineStyle = wdLineStyleDouble
    End With
  Else
    Set catTable = target.Tables(1)
  End If
  
  Do While catTable.Rows.Count <> 1
    Call catTable.Rows.Last.Delete
  Loop
End Function

Private Function OutputCategories(target As Word.Table)
  Dim outDoc As Word.Document: Set outDoc = target.Range.Document
  Dim iCat As InfoCategory
  For Each iCat In categories_
    Dim newRow As Word.Row: Set newRow = target.Rows.Add
    With newRow
      .Cells(WORD_CAT_ID).Range = iCat.id_
      .Cells(WORD_CAT_NAME).Range = iCat.name_
      .Cells(WORD_CAT_STYLE).Range = iCat.style_
      .Cells(WORD_CAT_INFO).Range = iCat.info_
      .Cells(WORD_CAT_COMP).Range = iCat.GetCompositionString
      If iCat.IsComplete Then _
        .Cells(WORD_CAT_STYLE).Range.Style = iCat.style_
    End With
  Next iCat
End Function

Private Function CreateStyleCombination(sStyle$, iCombo As Collection, iDoc As Word.Document) As Word.Style
  If WordStyleExists(iDoc, sStyle) Then _
    Exit Function
  
  Dim iFormat As New InfoFormat
  Dim iArgFormat As InfoFormat
  Dim iArg As Word.Style
  For Each iArg In iCombo
    Set iArgFormat = New InfoFormat: Call iArgFormat.InitFromWordDiff(iDoc.Styles(wdStyleNormal), iArg)
    If Not iFormat.IsCompatibleWith(iArgFormat) Then _
      Exit Function
    Call iFormat.MergeWith(iArgFormat)
  Next iArg
  
  Dim sResult As Word.Style: Set sResult = iDoc.Styles.Add(sStyle$, wdStyleTypeCharacter)
  Call iFormat.ApplyToWordFont(sResult.Font)
  Set CreateStyleCombination = sResult
End Function

Private Function UpdateHash()
  hash_ = 1
  Dim nCount&: nCount = 1
  
  Dim catID As Variant
  Dim iCat As InfoCategory
  For Each catID In categories_
    Set iCat = categories_(catID)
    hash_ = (hash_ + HashName(iCat.style_) + nCount + HashName(iCat.id_)) Mod 69208103 * 31&
    nCount = nCount + 1
  Next catID
End Function

Private Function HashName(sName$) As Long
  HashName = 1
  Dim nChar&
  For nChar = 1 To VBA.Len(sName)
    HashName = (HashName + VBA.AscW(VBA.Mid(sName, nChar, 1))) Mod 69208103 * 31&
  Next nChar
End Function
