VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "InfoDocument"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

Private Enum PuncType
  PT_NONE = 0
  PT_END = 1
  PT_CONTINUATION = 2
End Enum

Public doc_ As Word.Document
Public markup_ As MarkupAPI

Public Function Init(iDoc As Word.Document, iMarkup As MarkupAPI)
  Set doc_ = iDoc
  Set markup_ = iMarkup
End Function

Public Function SyncStyles()
  Call markup_.SyncWord(doc_)
End Function

Public Function GetFormatIterator(catID&, Optional nStart& = 0) As IteratorFormat
'      ,   
  If Not markup_.config_.Contains(catID) Then _
    Exit Function
  Dim iCat As InfoCategory: Set iCat = markup_.config_.GetAt(catID)
  If Not iCat.IsComplete Then _
    Exit Function
  If nStart > doc_.Range.End Then _
    Exit Function
  
  Set GetFormatIterator = New IteratorFormat
  Call GetFormatIterator.Init(doc_, iCat.GetFormat(doc_), nStart)
End Function

Public Function GetPreciseIterator(catID&, Optional nStart& = 0) As IteratorStyle
'      
  If Not markup_.config_.Contains(catID) Then _
    Exit Function
  Dim iCat As InfoCategory: Set iCat = markup_.config_.GetAt(catID)
  If Not iCat.IsComplete Then _
    Exit Function
  If nStart > doc_.Range.End Then _
    Exit Function
  
  Dim cStyles As New Collection
  Call cStyles.Add(iCat.style_)
  Set GetPreciseIterator = New IteratorStyle
  Call GetPreciseIterator.Init(doc_, cStyles, nStart)
End Function

Public Function GetStylesIterator(catID&, Optional nStart& = 0) As IteratorStyle
'      
  If Not markup_.config_.Contains(catID) Then _
    Exit Function
  Dim iCat As InfoCategory: Set iCat = markup_.config_.GetAt(catID)
  If Not iCat.IsComplete Then _
    Exit Function
  If nStart > doc_.Range.End Then _
    Exit Function
  
  Dim cStyles As New Collection
  Call cStyles.Add(iCat.style_)
  Dim aCat As InfoCategory
  For Each aCat In markup_.config_.GetChildrenOf(catID)
    Call cStyles.Add(aCat.style_)
  Next aCat
  
  Set GetStylesIterator = New IteratorStyle
  Call GetStylesIterator.Init(doc_, cStyles, nStart)
End Function

Public Function ApplyCategory(iCat As InfoCategory, ByRef target As Word.Range) As Boolean
  ApplyCategory = False

  If Not iCat.IsComplete Then _
    Exit Function
  target.Style = iCat.style_
  
  ApplyCategory = True
End Function

Public Function ApplyFragments(iFragments As Collection) As Boolean
  ApplyFragments = False
  
  Dim iFrag As ItemFragment
  For Each iFrag In iFragments
    If Not markup_.config_.Contains(iFrag.id_) Then _
      Exit Function
    If Not ApplyCategory(markup_.config_.GetAt(iFrag.id_), doc_.Range(iFrag.start_, iFrag.end_)) Then _
      Exit Function
  Next iFrag

  ApplyFragments = True
End Function

Public Function AddCategory(iAddedCat As InfoCategory, ByRef target As Word.Range) As Boolean
  AddCategory = False
  
  If Not iAddedCat.IsComplete Then _
    Exit Function
    
  Dim sCurrentStyle$: sCurrentStyle = target.Style.NameLocal
  If sCurrentStyle = iAddedCat.style_ Then
    AddCategory = True
    Exit Function
  End If
  
  Dim iCurrentCat As InfoCategory: Set iCurrentCat = markup_.config_.FindStyle(sCurrentStyle)
  If iCurrentCat Is Nothing Then
    target.Style = iAddedCat.style_
    AddCategory = True
    Exit Function
  ElseIf iCurrentCat.composition_.Exists(iAddedCat.id_) Then
    AddCategory = True
    Exit Function
  End If
  
  Dim iComposition As Scripting.Dictionary:  Set iComposition = CSet(iAddedCat.id_, iCurrentCat.id_)
  Dim iCombo As InfoCategory: Set iCombo = markup_.config_.FindComposition(iComposition)
  If Not iCombo Is Nothing Then
    target.Style = iCombo.style_
    AddCategory = True
    Exit Function
  End If
End Function

Public Function SetCategoryVisible(catID&, bShow As Boolean)
  Dim iter As IteratorFormat: Set iter = GetFormatIterator(catID)
  If iter Is Nothing Then _
    Exit Function
  Do While Not iter.MoveNext Is Nothing
    Dim rFragment As Word.Range: Set rFragment = FixFragment(iter.Range)
    rFragment.Font.Hidden = Not bShow
  Loop
End Function

Public Function SetCategoryHighlight(catID&, bHighlight As Boolean)
  Dim iter As IteratorFormat: Set iter = GetFormatIterator(catID)
  If iter Is Nothing Then _
    Exit Function
  Do While Not iter.MoveNext Is Nothing
    Dim rFragment As Word.Range: Set rFragment = iter.Range
    rFragment.HighlightColorIndex = IIf(bHighlight, wdYellow, wdNoHighlight)
  Loop
End Function

Public Function GetAllHighlights() As Collection ' of ItemFragment
  Set GetAllHighlights = New Collection
  Dim rFind As Word.Range: Set rFind = doc_.Range(0, 0)
  With rFind.Find
    .ClearFormatting
    .Highlight = True
  End With
  With rFind
    Do While .Find.Execute
      Dim aFrag As New ItemFragment
      aFrag.id_ = .HighlightColorIndex
      aFrag.start_ = .Start
      aFrag.end_ = .End
      Call GetAllHighlights.Add(aFrag.Clone)
    Loop
  End With
End Function

Public Function GetBasicFragmentsCompound() As Collection ' of ItemFragment
  Dim catIntervals As New Scripting.Dictionary
  
  Dim iCat As InfoCategory
  Dim catID As Variant
  For Each catID In markup_.config_.categories_
    Set iCat = markup_.config_.categories_(catID)
    If Not iCat.IsCombo Then _
      Set catIntervals.Item(catID) = New CDS_CompoundIntervals
  Next catID
  
  Dim allFrags As Collection: Set allFrags = GetAllFragments()
  Dim aFrag As ItemFragment
  For Each aFrag In allFrags
    Set iCat = markup_.config_.GetAt(aFrag.id_)
    If Not iCat.IsCombo Then
      Call catIntervals(iCat.id_).AddItem(aFrag.start_, aFrag.end_)
    Else
      Dim comboItem As Variant
      For Each comboItem In iCat.composition_
        Call catIntervals(comboItem).AddItem(aFrag.start_, aFrag.end_)
      Next comboItem
    End If
  Next aFrag
  
  Dim cResult As New Collection
  For Each catID In markup_.config_.categories_
    Set iCat = markup_.config_.categories_(catID)
    If Not iCat.IsCombo Then
      Dim interval As CDS_Interval
      For Each interval In catIntervals(catID).items_
        Set aFrag = New ItemFragment
        aFrag.id_ = catID
        aFrag.start_ = interval.start_
        aFrag.end_ = interval.finish_
        Call cResult.Add(aFrag)
      Next interval
    End If
  Next catID
  
  Set GetBasicFragmentsCompound = cResult
End Function

Public Function GetAllFragments() As Collection ' of ItemFragment
  Set GetAllFragments = New Collection
  Dim catID As Variant
  For Each catID In markup_.config_.categories_
    Call InternalFragments(GetAllFragments, CLng(catID))
  Next catID
End Function

' ============
Private Function InternalFragments(ByRef out As Collection, catID&)
  Dim aFrag As New ItemFragment
  aFrag.id_ = catID
  Dim iter As IteratorStyle: Set iter = GetPreciseIterator(catID)
  Do While Not iter.MoveNext Is Nothing
    aFrag.start_ = iter.Range.Start
    aFrag.end_ = iter.Range.End
    Call out.Add(aFrag.Clone)
  Loop
End Function

Private Function FixFragment(target As Word.Range) As Word.Range
  Dim rCurrent As Word.Range: Set rCurrent = WordAdjustRange(target)
  Set FixFragment = rCurrent ' Note: alias, not copy!
  
  Dim rightB As ItemFragment: Set rightB = GetRightBoundary(target)
  Dim leftB As ItemFragment: Set leftB = GetLeftBoundary(target)
  If rightB Is Nothing Or leftB Is Nothing Then _
    Exit Function
  
  If rightB.id_ = PT_NONE Or leftB.id_ = PT_NONE Then
    If rightB.id_ = leftB.id_ Then
      rCurrent.Start = leftB.start_
      rCurrent.End = rightB.end_
    End If
  ElseIf leftB.id_ = PT_END Then
    rCurrent.End = rightB.end_
  ElseIf leftB.id_ = rightB.id_ Then
    rCurrent.Start = leftB.start_
    rCurrent.End = rightB.end_
  Else
    rCurrent.Start = leftB.start_
  End If
End Function

Private Function GetLeftBoundary(target As Word.Range) As ItemFragment
  If target.Start = 0 Then _
    Exit Function
  
  Dim tRange As Word.Range: Set tRange = target.Duplicate
  Call tRange.Collapse(wdCollapseStart)
  Call tRange.MoveStart(wdWord, -1)
  Dim Result As New ItemFragment
  Result.end_ = tRange.End
  Result.start_ = tRange.Start
  
  Dim sText$: sText = Trim(tRange.Text)
  If sText Like "[" + Endings + "]" Then
    Result.id_ = PT_END
    Set GetLeftBoundary = Result
    Exit Function
  End If
  
  If sText Like "[" + Continuations + "]" Then
    Result.id_ = PT_CONTINUATION
    Set GetLeftBoundary = Result
    Exit Function
  End If
  
  If sText = "(" Then
    Result.id_ = PT_NONE
    Set GetLeftBoundary = Result
    Exit Function
  End If
End Function

Private Function GetRightBoundary(target As Word.Range) As ItemFragment
  If target.End = target.Document.Range.End Then _
    Exit Function
    
  Dim tRange As Word.Range: Set tRange = target.Duplicate
  Call tRange.Collapse(wdCollapseEnd)
  Call tRange.MoveEnd(wdWord, 1)
  Dim Result As New ItemFragment
  Result.end_ = tRange.End
  Result.start_ = tRange.Start
  
  Dim sText$: sText = Trim(tRange.Text)
  If sText Like "[" + Endings + "]" Then
    Result.id_ = PT_END
    Set GetRightBoundary = Result
    Exit Function
  End If
  
  If sText Like "[" + Continuations + "]" Then
    Result.id_ = PT_CONTINUATION
    Set GetRightBoundary = Result
    Exit Function
  End If
  
  If sText = ")" Then
    Dim nextRight As ItemFragment: Set nextRight = GetRightBoundary(tRange)
    If Not nextRight Is Nothing Then
      If nextRight.id_ = PT_END Then
        Result.id_ = PT_END
        Result.end_ = nextRight.end_
        Set GetRightBoundary = Result
        Exit Function
      End If
    End If
    Result.id_ = PT_NONE
    Set GetRightBoundary = Result
    Exit Function
  End If
End Function

Private Function Continuations() As String
  Continuations = ",;:-" & Chr(150) & Chr(151)
End Function

Private Function Endings() As String
  Endings = ".?!" & Chr(13)
End Function
