VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "InfoFormat"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
'================       =========================
Option Explicit

Public Enum TFormatFeature
  T_FF_INVALID = 0
  [_First] = 1
  T_FF_FONT_NAME = 1
  T_FF_FONT_SIZE
  T_FF_TEXT_COLOR
  T_FF_BG_COLOR
  T_FF_BOLD
  T_FF_STRIKETHROUGH
  T_FF_ITALIC
  T_FF_UNDERLINE
  T_FF_UNDERLINE_COLOR
  T_FF_ALLCAPS
  [_Last] = 10
End Enum

Public features_ As Scripting.Dictionary

Private Sub Class_Initialize()
  Set features_ = New Scripting.Dictionary
End Sub

Public Function Clone() As InfoFormat
  Set Clone = New InfoFormat
  Dim aKey As Variant
  For Each aKey In features_
    Call Clone.features_.Add(aKey, features_(aKey))
  Next aKey
End Function

Public Function InitFromWord(wStyle As Word.Style)
  Call ClearFeatures
  
  Dim nFeature&
  For nFeature = TFormatFeature.[_First] To TFormatFeature.[_Last]
    Call features_.Add(nFeature, WordGetValue(nFeature, wStyle.Font))
  Next nFeature
End Function

Public Function InitFromWordDiff(sBase As Word.Style, sDerived As Word.Style)
  Call ClearFeatures
  
  Dim nFeature&
  For nFeature = TFormatFeature.[_First] To TFormatFeature.[_Last]
    If WordGetValue(nFeature, sBase.Font) <> WordGetValue(nFeature, sDerived.Font) Then _
      Call features_.Add(nFeature, WordGetValue(nFeature, sDerived.Font))
  Next nFeature
End Function

Public Function IsEmpty() As Boolean
  IsEmpty = features_.Count = 0
End Function

Public Function ApplyToWordFont(ByRef target As Word.Font)
  Dim nFeature As Variant
  For Each nFeature In features_
    Call WordSetValue(CLng(nFeature), target, features_(nFeature))
  Next nFeature
End Function

Public Function Compare(rValue As InfoFormat) As Double
  Compare = CompareDeep(features_, rValue.features_)
End Function

Public Function IsCompatibleWith(rValue As InfoFormat) As Boolean
  IsCompatibleWith = False
  
  Dim nFeature As Variant
  For Each nFeature In features_
    If rValue.features_.Exists(nFeature) Then _
      If features_(nFeature) <> rValue.features_(nFeature) Then _
        Exit Function
  Next nFeature
  
  IsCompatibleWith = True
End Function

' !Requires IsCompatibleWith(rValue)
Public Function MergeWith(rValue As InfoFormat)
  Dim nFeature As Variant
  For Each nFeature In rValue.features_
    features_.Item(nFeature) = rValue.features_(nFeature)
  Next nFeature
End Function

' =========
Private Function ClearFeatures()
  Call features_.RemoveAll
End Function

Private Function WordGetValue(nFeature As TFormatFeature, wFont As Word.Font) As Variant
  Select Case nFeature
    Case T_FF_FONT_NAME: WordGetValue = wFont.Name
    Case T_FF_FONT_SIZE: WordGetValue = wFont.Size
    Case T_FF_BOLD: WordGetValue = wFont.Bold
    Case T_FF_ITALIC: WordGetValue = wFont.Italic
    Case T_FF_TEXT_COLOR: WordGetValue = wFont.TextColor
    Case T_FF_BG_COLOR: WordGetValue = wFont.Shading.BackgroundPatternColor
    Case T_FF_UNDERLINE_COLOR: WordGetValue = wFont.UnderlineColor
    Case T_FF_STRIKETHROUGH: WordGetValue = wFont.StrikeThrough
    Case T_FF_UNDERLINE: WordGetValue = wFont.Underline
    Case T_FF_ALLCAPS: WordGetValue = wFont.AllCaps
  End Select
End Function

Private Function WordSetValue(nFeature As TFormatFeature, ByRef target As Word.Font, iValue As Variant)
  Select Case nFeature
    Case T_FF_FONT_NAME: target.Name = iValue
    Case T_FF_FONT_SIZE: target.Size = iValue
    Case T_FF_BOLD: target.Bold = iValue
    Case T_FF_ITALIC: target.Italic = iValue
    Case T_FF_TEXT_COLOR: target.TextColor = iValue
    Case T_FF_BG_COLOR: target.Shading.BackgroundPatternColor = iValue
    Case T_FF_UNDERLINE_COLOR: target.UnderlineColor = iValue
    Case T_FF_STRIKETHROUGH: target.StrikeThrough = iValue
    Case T_FF_UNDERLINE: target.Underline = iValue
    Case T_FF_ALLCAPS: target.AllCaps = iValue
  End Select
End Function

