Attribute VB_Name = "Main"
Option Explicit

Public Sub MU_OpenConfig()
  If Not PreloadConfig Then _
    Exit Sub
  If Not UserInteraction.AskQuestion(QM_CONFIG_EDIT_CONFIRM) Then _
    Exit Sub
  Dim iShell As New Shell32.Shell
  Call iShell.Open(Markup.config_.sourcePath_)
End Sub

Public Sub MU_SyncStyles()
  If Not PreloadConfig Then _
    Exit Sub
  Call Markup.config_.SyncStyles(ThisDocument.Application, ActiveDocument.FullName)
  Call UserInteraction.ShowMessage(IM_STYLES_UPDATE_SUCCESS)
End Sub

Public Sub MU_LoadConfig()
  Dim sFile$: sFile = UserInteraction.PromptFile(ActiveDocument.Path)
  If sFile = vbNullString Then _
    Exit Sub
  Call MU_LoadConfigFrom(sFile)
  Call UIUpdateConfigList
End Sub

Public Function MU_LoadConfigFrom(sFile$) As Boolean
  If Not Markup.config_.Load(sFile) Then
    Call UserInteraction.ShowMessage(EM_EXPORT_CONFIG_FAIL)
    MU_LoadConfigFrom = False
  Else
    Call UIResetCategories
    MU_LoadConfigFrom = True
  End If
End Function

Public Sub MU_DetermineCategory()
 If Not PreloadConfig Then _
    Exit Sub
    
  Dim iCat As InfoCategory: Set iCat = Markup.config_.FindStyle(Selection.Style)
  If iCat Is Nothing Then
    Call UserInteraction.ShowMessage(IM_NO_CATEGORY)
  Else
    Call UserInteraction.ShowMessage(IM_CATEGORY_OUT, iCat.id_, iCat.name_)
  End If
End Sub

Public Sub MU_ApplyCategory()
  If Not PreloadConfig Then _
    Exit Sub
  Dim iCat As InfoCategory: Set iCat = ActiveCategory
  If iCat Is Nothing Then _
    Exit Sub
  
  Dim iDoc As InfoDocument: Set iDoc = GetActiveWrapper
  Dim rSelection As Word.Range: Set rSelection = WordAdjustRange(Selection.Range)
  If iDoc.ApplyCategory(iCat, rSelection) Then _
    Call rSelection.Select
End Sub

Public Sub MU_AddCategory()
  If Not PreloadConfig Then _
    Exit Sub
  Dim iCat As InfoCategory: Set iCat = ActiveCategory
  If iCat Is Nothing Then _
    Exit Sub
  
  Dim iDoc As InfoDocument: Set iDoc = GetActiveWrapper
  Dim rSelection As Word.Range: Set rSelection = WordAdjustRange(Selection.Range)
  If iDoc.AddCategory(iCat, rSelection) Then
    Call rSelection.Select
    Call UIUpdateCategories
  Else
    Call UserInteraction.ShowMessage(EM_CANNOT_COMBINE_STYLES)
  End If
End Sub

Public Sub MU_ClearFragment()
  Dim tRange As Word.Range: Set tRange = Selection.Range.Duplicate
  Dim iDoc As New InfoDocument:  Call iDoc.Init(ActiveDocument, Markup)
  Set tRange = WordAdjustRange(tRange)
  tRange.Style = wdStyleDefaultParagraphFont
  Call tRange.Select
End Sub

Sub MU_NextCategory()
  If Not PreloadConfig Then _
    Exit Sub
  Call UINextCat
End Sub

Sub MU_PrevCategory()
  If Not PreloadConfig Then _
    Exit Sub
  Call UIPrevCat
End Sub

Public Sub MU_GotoNext()
  If Not PreloadConfig Then _
    Exit Sub
  Dim iCat As InfoCategory: Set iCat = ActiveCategory
  If iCat Is Nothing Then _
    Exit Sub
  
  Dim iter As IteratorFormat: Set iter = GetActiveWrapper.GetFormatIterator(iCat.id_, Selection.End)
  Dim rFound As Word.Range: Set rFound = iter.MoveNext
  If Not rFound Is Nothing Then _
    Call rFound.Select
End Sub

Public Sub MU_GotoPrev()
  If Not PreloadConfig Then _
    Exit Sub
  Dim iCat As InfoCategory: Set iCat = ActiveCategory
  If iCat Is Nothing Then _
    Exit Sub
  
  Dim iter As IteratorFormat: Set iter = GetActiveWrapper.GetFormatIterator(iCat.id_, Selection.Start)
  Dim rFound As Word.Range: Set rFound = iter.MovePrev
  If Not rFound Is Nothing Then _
    Call rFound.Select
End Sub

Public Sub MU_HideText()
  If Not PreloadConfig Then _
    Exit Sub
  Dim iCat As InfoCategory: Set iCat = ActiveCategory
  If iCat Is Nothing Then _
    Exit Sub
  
  Dim wUI As New API_WordWrapper: Call wUI.SetDocument(ActiveDocument)
  Call wUI.PauseUI
  Call GetActiveWrapper.SetCategoryVisible(iCat.id_, False)
  Call wUI.ResumeUI
End Sub

Public Sub MU_ShowText()
  If Not PreloadConfig Then _
    Exit Sub
  Dim iCat As InfoCategory: Set iCat = ActiveCategory
  If iCat Is Nothing Then _
    Exit Sub
  
  Dim wUI As New API_WordWrapper: Call wUI.SetDocument(ActiveDocument)
  Call wUI.PauseUI
  Call GetActiveWrapper.SetCategoryVisible(iCat.id_, True)
  Call wUI.ResumeUI
End Sub

Public Sub MU_HighlightText()
  If Not PreloadConfig Then _
    Exit Sub
  Dim iCat As InfoCategory: Set iCat = ActiveCategory
  If iCat Is Nothing Then _
    Exit Sub
  
  Dim wUI As New API_WordWrapper: Call wUI.SetDocument(ActiveDocument)
  Call wUI.PauseUI
  Call GetActiveWrapper.SetCategoryHighlight(iCat.id_, True)
  Call wUI.ResumeUI
End Sub

Public Sub MU_UnhighlightText()
  If Not PreloadConfig Then _
    Exit Sub
  Dim iCat As InfoCategory: Set iCat = ActiveCategory
  If iCat Is Nothing Then _
    Exit Sub
  
  Dim wUI As New API_WordWrapper: Call wUI.SetDocument(ActiveDocument)
  Call wUI.PauseUI
  Call GetActiveWrapper.SetCategoryHighlight(iCat.id_, False)
  Call wUI.ResumeUI
End Sub

Public Sub MU_ExtractVisible()
  Dim iSource As Word.Document: Set iSource = ActiveDocument
  
  Dim wrapper As New API_WordWrapper
  Dim iDestination As Word.Document: Set iDestination = wrapper.NewDocument
  If iDestination Is Nothing Then _
    Exit Sub
    
  Call TransferVisible(iSource, iDestination)
  
  Call UserInteraction.ShowMessage(IM_EXPORT_VISIBLE_SUCCESS)
End Sub

Public Sub MU_ExtractCategory()
  If Not PreloadConfig Then _
    Exit Sub

  Dim iCat As InfoCategory: Set iCat = ActiveCategory
  If iCat Is Nothing Then _
    Exit Sub
  
  Dim wrapper As New API_WordWrapper
  Dim iDestination As Word.Document: Set iDestination = wrapper.NewDocument
  If iDestination Is Nothing Then _
    Exit Sub
  
  Dim iter As IteratorFormat: Set iter = GetActiveWrapper.GetFormatIterator(iCat.id_)
  Do While Not iter.MoveNext Is Nothing
    Call iter.Range.Copy
    Call iDestination.Range.InsertAfter(vbNewLine)
    Call iDestination.Paragraphs.Last.Range.Paste
  Loop
  
  Call UserInteraction.ShowMessage(IM_EXPORT_CATEGORY_SUCCESS)
End Sub

Public Sub MU_ExportMarkup()
  If Not PreloadConfig Then _
    Exit Sub
  
  Dim iDoc As InfoDocument: Set iDoc = GetActiveWrapper

  Dim outName$: outName = ActiveDocument.FullName & "_export" & ".txt"
  Dim fso As New Scripting.FileSystemObject
  Dim textOut As Object: Set textOut = fso.CreateTextFile(outName, True)

  Dim frags As Collection: Set frags = iDoc.GetAllFragments()
  Dim aFrag As ItemFragment
  For Each aFrag In frags
    Call textOut.WriteLine(CStr(aFrag.start_) & vbTab & CStr(aFrag.end_) & vbTab & aFrag.id_)
  Next aFrag
  
  Call textOut.Close
  
  Dim oShell As New Shell32.Shell
  Call oShell.Open(outName)
End Sub

Public Sub MU_ImportMarkup()
  If Not PreloadConfig Then _
    Exit Sub
    
  Dim sFile$: sFile = UserInteraction.PromptFile(ActiveDocument.Path)
  If sFile = "" Then _
    Exit Sub
  Dim fragments As Collection: Set fragments = ScanFragmentsFrom(sFile)
  If fragments Is Nothing Then _
    Exit Sub
  
  Dim iDoc As InfoDocument: Set iDoc = GetActiveWrapper
  If iDoc.ApplyFragments(fragments) Then
    Call UserInteraction.ShowMessage(IM_IMPORT_SUCCESS)
  Else
    Call UserInteraction.ShowMessage(EM_IMPORT_FAIL)
  End If
End Sub

Public Sub MU_ExportMarkedText()
  If Not PreloadConfig Then _
    Exit Sub
  
  Dim iDoc As InfoDocument: Set iDoc = GetActiveWrapper
  
  Dim xlApp As New API_XLWrapper
  Dim outWB As Excel.Workbook: Set outWB = xlApp.NewDocument
  If outWB Is Nothing Then
    Call UserInteraction.ShowMessage(EM_FAILED_OUTPUT_CREATION)
    Exit Sub
  End If
  
  Dim frags As Collection: Set frags = iDoc.GetBasicFragmentsCompound()
  
  Call CSE_ProgressBar.Init(" ", maxVal:=frags.Count)
  Call CSE_ProgressBar.Show

  Call xlApp.PauseUI
  Call Output2Excel(outWB, frags)
  Call xlApp.ResumeUI
  
  Call Unload(CSE_ProgressBar)
  
  Call UserInteraction.ShowMessage(IM_EXCEL_OUT_SUCCESS)
End Sub

Public Sub MU_ExportHighlights()
  Dim outName$: outName = ActiveDocument.FullName & "_export" & ".txt"
  Dim fso As New Scripting.FileSystemObject
  Dim textOut As Object: Set textOut = fso.CreateTextFile(outName, True)
  
  Dim iDoc As New InfoDocument: Call iDoc.Init(ActiveDocument, Markup)
  Dim frags As Collection: Set frags = iDoc.GetAllHighlights()
  Dim aFrag As ItemFragment
  For Each aFrag In frags
    Call textOut.WriteLine(CStr(aFrag.start_) & vbTab & CStr(aFrag.end_) & vbTab & aFrag.id_)
  Next aFrag
  
  Call textOut.Close
  
  Dim oShell As New Shell32.Shell
  Call oShell.Open(outName)
End Sub

Public Sub MU_ViewHelp()
  Dim aWrap As New API_WordWrapper
  Dim helpDoc As Word.Document: Set helpDoc = aWrap.OpenDocument(APP_HELP, bReadOnly:=True)
  If helpDoc Is Nothing Then _
    Exit Sub
    
  helpDoc.ActiveWindow.View.ReadingLayout = False
  Call helpDoc.Activate
End Sub

' =======
Private Function GetActiveWrapper() As InfoDocument
  Dim iDoc As New InfoDocument: Call iDoc.Init(ActiveDocument, Markup)
  Call iDoc.SyncStyles
  Set GetActiveWrapper = iDoc
End Function

Private Function PreloadConfig() As Boolean
  PreloadConfig = False
  
  If Not Markup.config_.IsLoaded Then
    If Not Markup.InitFromCurrentDocument(ActiveDocument) Then
      Call UserInteraction.ShowMessage(EM_CONFIG_LOAD_FAIL)
      Exit Function
    Else
      Call UIUpdateConfigList
      Call UIResetCategories
    End If
  End If
  
  PreloadConfig = True
End Function
