Attribute VB_Name = "z_UIMessages"
' Messaging module
Option Private Module
Option Explicit

Public Enum MsgCode
  EM_COMBO_TAKEN = ERR_COMBO_TAKEN
  EM_CANNOT_ADD_STYLE = ERR_CANNOT_ADD_STYLE
  EM_STYLE_TAKEN = ERR_STYLE_TAKEN
  EM_FORMAT_TAKEN = ERR_FORMAT_TAKEN
  EM_UNKNOWN_CAT_ID = ERR_UNKNOWN_CAT_ID
  EM_CAT_NAME_TAKEN = ERR_CAT_NAME_TAKEN
  EM_STYLE_NO_FORMAT = ERR_STYLE_NO_FORMAT
  EM_INCOMPLETE_COMPOSITION = ERR_INCOMPLETE_COMPOSITION

  EM_EXPORT_CONFIG_FAIL = 1
  EM_FAILED_OUTPUT_CREATION
  EM_IMPORT_FAIL
  EM_CONFIG_LOAD_FAIL
  EM_CANNOT_COMBINE_STYLES
  EM_CONFIG_MISSING
  EM_ID_NOT_SELECTED
  EM_STYLE_NOT_SELECTED
  EM_CATEGORY_NOT_SELECTED
  EM_RIBBON_NOT_REACHABLE
  EM_VALIDATION_FAILED
  EM_CATEGORY_INVALID
  EM_COMBO_VALIDATION_FAILED
  EM_COMBO_MISSING_TEXT
 
  IM_EXPORT_VISIBLE_SUCCESS
  IM_EXPORT_CATEGORY_SUCCESS
  IM_IMPORT_SUCCESS
  IM_EXCEL_OUT_SUCCESS
  IM_STYLES_UPDATE_SUCCESS
  IM_NO_CATEGORY
  IM_CATEGORY_OUT
  
  QM_CONFIG_EDIT_CONFIRM
End Enum

Private g_UI As API_UserInteraction

Public Function UserInteraction() As API_UserInteraction
  If g_UI Is Nothing Then _
    Set g_UI = New API_UserInteraction
  Set UserInteraction = g_UI
End Function

Public Function SetUserInteraction(newUI As API_UserInteraction)
  Set g_UI = newUI
End Function

Public Function ProcessErrorMessages(expectedErrors As Scripting.Dictionary)
  If Err.Number = 0 Then _
    Exit Function
    
  Call Unload(CSE_ProgressBar)
  If Not expectedErrors.Exists(Err.Number) Then _
    Call Err.Raise(Err.Number)
  
  Call UserInteraction.ShowMessage(Err.Number, Err.Source)
End Function

Public Function UIShowMessage(theCode As MsgCode, ParamArray params() As Variant)
  Dim unwrapped As Variant: unwrapped = params
  unwrapped = FixForwardedParams(unwrapped)
  
  Select Case theCode
    Case EM_EXPORT_CONFIG_FAIL:       Call MsgBox("     ", vbExclamation)
    Case EM_FAILED_OUTPUT_CREATION:   Call MsgBox("      ", vbExclamation)
    Case EM_IMPORT_FAIL:              Call MsgBox("      ", vbInformation)
    Case EM_CONFIG_LOAD_FAIL:         Call MsgBox("  ", vbExclamation)
    Case EM_CANNOT_COMBINE_STYLES:    Call MsgBox("  ", vbExclamation)
    Case EM_CONFIG_MISSING:           Call MsgBox(" ", vbExclamation)
    Case EM_ID_NOT_SELECTED:          Call MsgBox("  ", vbExclamation)
    Case EM_STYLE_NOT_SELECTED:       Call MsgBox(" !", vbExclamation)
    Case EM_CATEGORY_NOT_SELECTED:    Call MsgBox("  ", vbExclamation)
    Case EM_CATEGORY_INVALID:         Call MsgBox(" ,   ", vbExclamation)
    Case EM_VALIDATION_FAILED:        Call MsgBox("   ", vbExclamation)
    Case EM_COMBO_VALIDATION_FAILED:  Call MsgBox("    ", vbExclamation)
    Case EM_COMBO_MISSING_TEXT:       Call MsgBox("    ", vbExclamation)
    Case EM_COMBO_TAKEN:              Call MsgBox(Fmt("   .  : {1}", unwrapped), vbExclamation)
    Case EM_CANNOT_ADD_STYLE:         Call MsgBox("   ", vbExclamation)
    Case EM_STYLE_TAKEN:              Call MsgBox("     ", vbExclamation)
    Case EM_FORMAT_TAKEN:             Call MsgBox(Fmt("   .  : {1}", unwrapped), vbExclamation)
    Case EM_UNKNOWN_CAT_ID:           Call MsgBox("  ", vbExclamation)
    Case EM_CAT_NAME_TAKEN:           Call MsgBox(Fmt("  : {1}", unwrapped), vbExclamation)
    Case EM_STYLE_NO_FORMAT:          Call MsgBox(Fmt("     : {1}", unwrapped), vbExclamation)
    Case EM_INCOMPLETE_COMPOSITION:   Call MsgBox("   ", vbExclamation)
    Case EM_RIBBON_NOT_REACHABLE
      Call MsgBox("     !" & vbNewLine & _
                  ",    Word       ", vbExclamation)
      
    Case IM_EXPORT_VISIBLE_SUCCESS:   Call MsgBox("    ", vbInformation)
    Case IM_EXPORT_CATEGORY_SUCCESS:  Call MsgBox("     ", vbInformation)
    Case IM_IMPORT_SUCCESS:           Call MsgBox("  ", vbInformation)
    Case IM_EXCEL_OUT_SUCCESS:        Call MsgBox("  ", vbInformation)
    Case IM_STYLES_UPDATE_SUCCESS:    Call MsgBox(" ", vbInformation)
    Case IM_NO_CATEGORY:              Call MsgBox("      ", vbInformation)
    Case IM_CATEGORY_OUT:             Call MsgBox(Fmt("   : {1}", unwrapped), vbInformation)
      
    Case Else: Call MsgBox("  ", vbCritical)
  End Select
End Function

Public Function UIAskQuestion(theCode As MsgCode, ParamArray params() As Variant) As Boolean
  Dim unwrapped As Variant: unwrapped = params
  unwrapped = FixForwardedParams(unwrapped)
  
  Dim answer&: answer = vbNo
  Select Case theCode
  Case QM_CONFIG_EDIT_CONFIRM
    answer = MsgBox("!      .        " & _
      vbNewLine & "      ", vbYesNo + vbQuestion)
    
  Case Else
    Call MsgBox("Invalid message code", vbCritical)
  End Select
  UIAskQuestion = answer = vbYes
End Function
