Attribute VB_Name = "z_UIRibbon"
'      
Option Explicit

Global g_RibbonWrap As API_Ribbon

Global g_ActiveCat As Long
Global g_Configs As New Collection

Public Function RibbonTempFile() As String
  RibbonTempFile = Environ("TEMP") & "/" & TEMP_FILE_NAME & CStr(GetCurrentProcessId) & ".txt"
End Function

Public Function ActiveCategory() As InfoCategory
  If g_ActiveCat = 0 Then
    Call UserInteraction.ShowMessage(EM_CATEGORY_NOT_SELECTED)
    Exit Function
  End If
  
  Set ActiveCategory = Markup.config_.GetAt(g_ActiveCat)
  If ActiveCategory Is Nothing Then
    Call UserInteraction.ShowMessage(EM_CATEGORY_INVALID)
    Exit Function
  End If
End Function

Public Function UINextCat()
  If Markup.config_.categories_.Count = 0 Then _
    Exit Function
  
  Dim catID As Variant
  Dim nIndex&: nIndex = 0
  Dim ids() As Variant: ids = Markup.config_.categories_.Keys()
  For Each catID In ids
    If g_ActiveCat = catID Then
      If nIndex + 1 <> ArraySize(ids) Then
        g_ActiveCat = ids(nIndex + 1)
      Else
        g_ActiveCat = ids(0)
      End If
      Exit For
    End If
    nIndex = nIndex + 1
  Next catID
  
  Call UIUpdateCategories
End Function

Public Function UIPrevCat()
  Dim catID As Variant
  Dim nIndex&: nIndex = 0
  Dim ids() As Variant: ids = Markup.config_.categories_.Keys()
  For Each catID In ids
    If g_ActiveCat = catID Then
      If nIndex <> 0 Then
        g_ActiveCat = ids(nIndex - 1)
      Else
        g_ActiveCat = ids(ArraySize(ids) - 1)
      End If
      Exit For
    End If
    nIndex = nIndex + 1
  Next catID
  Call UIUpdateCategories
End Function

Public Function UIResetCategories()
  g_ActiveCat = 0
  If Markup.config_.IsLoaded Then
    If Markup.config_.categories_.Count > 0 Then _
      g_ActiveCat = Markup.config_.categories_.Keys(0)
  End If
  Call UIUpdateCategories
End Function

Public Function UIUpdateCategories()
  Dim aRibbon As IRibbonUI: Set aRibbon = GetRibbon
  If Not aRibbon Is Nothing Then _
    aRibbon.InvalidateControl ("CategoryPicker")
End Function

Public Function UIUpdateConfigList()
  Set g_Configs = GetConfigList
  
  Dim aRibbon As IRibbonUI: Set aRibbon = GetRibbon
  If Not aRibbon Is Nothing Then _
    Call aRibbon.InvalidateControl("ConfigPicker")
End Function

Public Sub MUC_LoadRibbon(aRibbon As IRibbonUI)
  Set g_RibbonWrap = New API_Ribbon
  Call g_RibbonWrap.Init(aRibbon, RibbonTempFile)
  Call UIUpdateConfigList
End Sub

Public Sub MUC_OnBtn(iControl As IRibbonControl)
  Call EnsureGlobalState
  Select Case iControl.ID
  Case "OpenConfig":          Call MU_OpenConfig
  
  Case "ApplyCategory":       Call MU_ApplyCategory
  Case "AddCategory":         Call MU_AddCategory
  Case "RemoveMarkup":        Call MU_ClearFragment
  
  Case "NextCategory":        Call MU_NextCategory
  Case "PrevCategory":        Call MU_PrevCategory

  Case "GetCategory":         Call MU_DetermineCategory
  Case "NextFragment":        Call MU_GotoNext
  Case "PreviousFragment":    Call MU_GotoPrev
  
  Case "HideCategory":        Call MU_HideText
  Case "ShowCategory":        Call MU_ShowText
  
  Case "HighlightCategory":   Call MU_HighlightText
  Case "UnhighlightCategory": Call MU_UnhighlightText
  
  Case "ExtractVisible":      Call MU_ExtractVisible
  Case "ExtractCategory":     Call MU_ExtractCategory
  Case "ExportMarkup":        Call MU_ExportMarkup
  Case "ImportMarkup":        Call MU_ImportMarkup
  Case "ExportMarkedText":    Call MU_ExportMarkedText
  Case "ExportHighlights":    Call MU_ExportHighlights
  Case "SyncStyles":          Call MU_SyncStyles
  Case "ViewHelp":            Call MU_ViewHelp
  End Select
End Sub

Public Sub MUC_ConfigCount(control As IRibbonControl, ByRef nCount)
  nCount = g_Configs.Count
End Sub

Public Sub MUC_ConfigLabel(control As IRibbonControl, nIndex%, ByRef vLabel)
  vLabel = g_Configs.Item(nIndex + 1).name_
End Sub

Public Sub MUC_ConfigSelected(control As IRibbonControl, ByRef val)
  Call EnsureGlobalState
  val = ActiveConfigIndex - 1
End Sub

Public Sub MUC_ConfigChange(control As IRibbonControl, sID$, nIndex As Variant)
  Call EnsureGlobalState
  If nIndex + 1 = ActiveConfigIndex Or nIndex = -1 Then _
    Exit Sub

  Dim sName$: sName = ActiveDocument.FullName
  If MU_LoadConfigFrom(g_Configs.Item(nIndex + 1).path_) Then
    Call Markup.config_.SyncStyles(ThisDocument.Application, sName)
  End If
End Sub

Sub MUC_CatsCount(control As IRibbonControl, ByRef nCount)
  nCount = Markup.config_.categories_.Count
End Sub

Public Sub MUC_CatsLabel(control As IRibbonControl, nIndex%, ByRef vLabel)
  vLabel = Markup.config_.categories_.Items(nIndex).name_
End Sub

Public Sub MUC_CatsSelected(control As IRibbonControl, ByRef nIndex)
  Call EnsureGlobalState
  
  nIndex = 0
  Dim catID As Variant
  For Each catID In Markup.config_.categories_
    If catID = g_ActiveCat Then _
      Exit Sub
    nIndex = nIndex + 1
  Next catID
  nIndex = -1
End Sub

Public Sub MUC_CatsChange(control As IRibbonControl, ID As String, nIndex As Variant)
  Call EnsureGlobalState
  If nIndex = -1 Then
    g_ActiveCat = vbNullString
  Else
    g_ActiveCat = Markup.config_.categories_.Keys(nIndex)
  End If
End Sub

' =============
Private Function EnsureGlobalState() As Boolean
  EnsureGlobalState = Not GetRibbon Is Nothing
End Function

Private Function ActiveConfigIndex() As Integer
  If Not Markup.config_.IsLoaded Then _
    Exit Function
    
  Dim nIndex&
  For nIndex = 1 To g_Configs.Count Step 1
    If g_Configs.Item(nIndex).path_ = Markup.config_.sourcePath_ Then
      ActiveConfigIndex = nIndex
      Exit Function
    End If
  Next nIndex
End Function

Private Function GetRibbon() As IRibbonUI
  If g_RibbonWrap Is Nothing Then
    Set g_RibbonWrap = New API_Ribbon
    Call g_RibbonWrap.LoadFrom(RibbonTempFile)
    Call UIUpdateConfigList
  End If
  Set GetRibbon = g_RibbonWrap.Value
  If GetRibbon Is Nothing Then _
    Call UserInteraction.ShowMessage(EM_RIBBON_NOT_REACHABLE)
End Function
