VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "s_Category"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

Private cat_ As InfoCategory

Public Function Setup()
  ' Mandatory setup function
  Set cat_ = New InfoCategory
End Function

Public Function Teardown()
  ' Mandatory teardown function
End Function

Public Function t_InitBasic()
  On Error GoTo PROPAGATE_ERROR
  
  Call cat_.InitBasic(1337, "name", "info")
  Call Dev_ExpectEQ(1337, cat_.id_)
  Call Dev_ExpectEQ("name", cat_.name_)
  Call Dev_ExpectEQ("info", cat_.info_)
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_Clone()
  On Error GoTo PROPAGATE_ERROR
  
  With cat_
    .id_ = 1337
    .name_ = "name1"
    .info_ = "info1"
    .style_ = "style1"
    Set .composition_ = CSet(1, 2, 3)
  End With
  
  Dim iClone As InfoCategory
  Set iClone = cat_.Clone
  
  Call Dev_NewCase("Copy data")
  Call Dev_ExpectEQ(cat_.id_, iClone.id_)
  Call Dev_ExpectEQ(cat_.name_, iClone.name_)
  Call Dev_ExpectEQ(cat_.info_, iClone.info_)
  Call Dev_ExpectEQ(cat_.style_, iClone.style_)
  Call Dev_ExpectEQ(cat_.composition_, iClone.composition_)
  
  With cat_
    .id_ = 1338
    .name_ = "name2"
    .info_ = "info2"
    .style_ = "style2"
    Call .composition_.Add(4, 0)
  End With
  
  Call Dev_NewCase("Deep copy")
  Call Dev_ExpectNE(cat_.id_, iClone.id_)
  Call Dev_ExpectNE(cat_.name_, iClone.name_)
  Call Dev_ExpectNE(cat_.info_, iClone.info_)
  Call Dev_ExpectNE(cat_.style_, iClone.style_)
  Call Dev_ExpectNE(cat_.composition_, iClone.composition_)
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_IsCombo()
  On Error GoTo PROPAGATE_ERROR
  
  Call Dev_ExpectFalse(cat_.IsCombo, "Empty category")
  
  Call cat_.composition_.Add(1, 0)
  Call Dev_ExpectFalse(cat_.IsCombo, "Single parent")
  
  Call cat_.composition_.Add(2, 0)
  Call Dev_ExpectTrue(cat_.IsCombo, "Valid combo")
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_IsComplete()
  On Error GoTo PROPAGATE_ERROR
  
  Call Dev_ExpectFalse(cat_.IsComplete, "Empty category")
  
  Call Dev_NewCase("Completely defined basic")
  With cat_
    .id_ = 1337
    .name_ = "name1"
    .info_ = "info1"
    .style_ = "style1"
  End With
  Call Dev_ExpectTrue(cat_.IsComplete)
  
  Call Dev_NewCase("Completely defined combo")
  Set cat_.composition_ = CSet(1, 2, 3)
  Call Dev_ExpectTrue(cat_.IsComplete)
  
  Call Dev_NewCase("Missing style")
  cat_.style_ = ""
  Call Dev_ExpectFalse(cat_.IsComplete)
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_ImportComposition()
  On Error GoTo PROPAGATE_ERROR
  
  Call Dev_ExpectTrue(cat_.ImportComposition(""), "Empty input")
  Call Dev_ExpectFalse(cat_.ImportComposition("invalid"), "Invalid id")
  
  Call Dev_NewCase("Single parent")
  Call Dev_ExpectTrue(cat_.ImportComposition("1"))
  Call Dev_ExpectEQ(1, cat_.composition_.Keys(0))
  
  Call Dev_NewCase("Valid input")
  Call Dev_ExpectTrue(cat_.ImportComposition("2; 3; 4"))
  Call Dev_ExpectEQ(CSet(2, 3, 4), cat_.composition_)
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_GetCompositionString()
  On Error GoTo PROPAGATE_ERROR
  
  Call Dev_ExpectEQ("", cat_.GetCompositionString, "Empty composition")
  
  Call Dev_NewCase("Single parent")
  Set cat_.composition_ = CSet(1)
  Call Dev_ExpectEQ("1", cat_.GetCompositionString)

  Call Dev_NewCase("Valid input")
  Set cat_.composition_ = CSet(1, 2, 3)
  Call Dev_ExpectEQ("1; 2; 3", cat_.GetCompositionString)
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_GetFormat()
  On Error GoTo PROPAGATE_ERROR
  
  Dim iBase As Word.Style: Set iBase = ThisDocument.Styles(wdStyleNormal)
  Dim iHeader As Word.Style: Set iHeader = ThisDocument.Styles(wdStyleHeading1)
  
  Call Dev_NewCase("Diff with base")
  Dim iEmpty As New InfoFormat
  cat_.style_ = iBase.NameLocal
  Dim iResult As InfoFormat: Set iResult = cat_.GetFormat(ThisDocument)
  Call Dev_ExpectTrue(iResult.Compare(iEmpty) = 0)
  
  Call Dev_NewCase("Diff with header")
  Dim iDiff As New InfoFormat: Call iDiff.InitFromWordDiff(iBase, iHeader)
  cat_.style_ = iHeader.NameLocal
  Set iResult = cat_.GetFormat(ThisDocument)
  Call Dev_ExpectTrue(iResult.Compare(iDiff) = 0)
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_IsIncludedIn()
  On Error GoTo PROPAGATE_ERROR
  
  cat_.id_ = 1
  Call Dev_ExpectFalse(cat_.IsIncludedIn(cat_), "Self not included")
  
  Call Dev_NewCase("Basic included")
  Dim iParent As New InfoCategory
  iParent.id_ = 42
  Call Dev_ExpectFalse(iParent.IsIncludedIn(cat_), "No connection")
  Set iParent.composition_ = CSet(1)
  Call Dev_ExpectFalse(iParent.IsIncludedIn(cat_))
  Call Dev_ExpectTrue(cat_.IsIncludedIn(iParent))
  
  Call Dev_NewCase("Complex included")
  Set cat_.composition_ = CSet(2, 3)
  Set iParent.composition_ = CSet(2, 3, 4)
  Call Dev_ExpectFalse(iParent.IsIncludedIn(cat_))
  Call Dev_ExpectTrue(cat_.IsIncludedIn(iParent))
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function
