VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "s_Format"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

Private format_ As InfoFormat

Public Function Setup()
  ' Mandatory setup function
  Set format_ = New InfoFormat
End Function

Public Function Teardown()
  ' Mandatory teardown function
End Function

Public Function t_IsEmpty()
  On Error GoTo PROPAGATE_ERROR
  
  Call Dev_ExpectTrue(format_.IsEmpty, "Empty")
  
  Call format_.features_.Add(T_FF_FONT_NAME, "test")
  Call Dev_ExpectFalse(format_.IsEmpty, "Not empty")

  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_InitFromWord()
  On Error GoTo PROPAGATE_ERROR
  
  Dim iStyle As Word.Style: Set iStyle = ThisDocument.Styles(wdStyleNormal)
  Call format_.InitFromWord(iStyle)
  Call Dev_ExpectFalse(format_.IsEmpty)
  Call Dev_AssertEQ(TFormatFeature.[_Last] - TFormatFeature.[_First] + 1, format_.features_.Count)
  Call Dev_ExpectEQ(iStyle.Font.Name, format_.features_(T_FF_FONT_NAME), "Font name")
  
  Call Dev_ExpectEQ(iStyle.Font.Size, format_.features_(T_FF_FONT_SIZE), "Font size")
  Call Dev_ExpectEQ(iStyle.Font.Bold, format_.features_(T_FF_BOLD), "Bold")
  Call Dev_ExpectEQ(iStyle.Font.Italic, format_.features_(T_FF_ITALIC), "Italic")
  Call Dev_ExpectEQ(iStyle.Font.TextColor, format_.features_(T_FF_TEXT_COLOR), "Text color")
  Call Dev_ExpectEQ(iStyle.Font.Shading.BackgroundPatternColor, format_.features_(T_FF_BG_COLOR), "Background color")
  Call Dev_ExpectEQ(iStyle.Font.UnderlineColor, format_.features_(T_FF_UNDERLINE_COLOR), "Underline color")
  Call Dev_ExpectEQ(iStyle.Font.StrikeThrough, format_.features_(T_FF_STRIKETHROUGH), "Strikethrough")
  Call Dev_ExpectEQ(iStyle.Font.Underline, format_.features_(T_FF_UNDERLINE), "Underline")
  Call Dev_ExpectEQ(iStyle.Font.AllCaps, format_.features_(T_FF_ALLCAPS), "All caps")

  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_InitFromWordDiff()
  On Error GoTo PROPAGATE_ERROR
  
  Dim iBase As Word.Style: Set iBase = ThisDocument.Styles(wdStyleNormal)
  Dim iHeader As Word.Style: Set iHeader = ThisDocument.Styles(wdStyleHeading1)

  Call Dev_NewCase("Self diff")
  Call format_.InitFromWordDiff(iBase, iBase)
  Call Dev_ExpectTrue(format_.IsEmpty)
  Call format_.InitFromWordDiff(iHeader, iHeader)
  Call Dev_ExpectTrue(format_.IsEmpty)
  
  Call Dev_NewCase("Valid diff")
  Call format_.InitFromWordDiff(iBase, iHeader)
  Call Dev_ExpectEQ(3, format_.features_.Count)
  
  Call Dev_ExpectEQ(iHeader.Font.Size, format_.features_(T_FF_FONT_SIZE), "Font size")
  Call Dev_ExpectEQ(iHeader.Font.Bold, format_.features_(T_FF_BOLD), "Bold")
  Call Dev_ExpectEQ(iHeader.Font.AllCaps, format_.features_(T_FF_ALLCAPS), "All caps")
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_ApplyToWordFont()
  On Error GoTo PROPAGATE_ERROR
  
  Dim iBase As Word.Style: Set iBase = ThisDocument.Styles(wdStyleNormal)
  Dim iHeader As Word.Style: Set iHeader = ThisDocument.Styles(wdStyleHeading1)
  Call format_.InitFromWordDiff(iBase, iHeader)
  
  Call Dev_NewCase("Apply to style font")
  Dim iTarget As Word.Style: Set iTarget = ThisDocument.Styles.Add("TestStyle", wdStyleTypeCharacter)
  iTarget.Font.Italic = True
  
  Call format_.ApplyToWordFont(iTarget.Font)
  Call Dev_ExpectEQ(format_.features_(T_FF_FONT_SIZE), iTarget.Font.Size, "Font size")
  Call Dev_ExpectEQ(format_.features_(T_FF_BOLD), iTarget.Font.Bold, "Bold")
  Call Dev_ExpectEQ(format_.features_(T_FF_ALLCAPS), iTarget.Font.AllCaps, "All caps")
  Call Dev_ExpectEQ(True, iTarget.Font.Italic, "Keep undef features")
  
  Call iTarget.Delete
  
  Call Dev_NewCase("Apply to range font")
  Dim rTarget As Word.Range: Set rTarget = ThisDocument.Range
  Call rTarget.InsertAfter("after")
  rTarget.Font.Italic = True
  
  Call format_.ApplyToWordFont(rTarget.Font)
  Call Dev_ExpectEQ(format_.features_(T_FF_FONT_SIZE), rTarget.Font.Size, "Font size")
  Call Dev_ExpectEQ(format_.features_(T_FF_BOLD), rTarget.Font.Bold, "Bold")
  Call Dev_ExpectEQ(format_.features_(T_FF_ALLCAPS), rTarget.Font.AllCaps, "All caps")
  Call Dev_ExpectEQ(True, rTarget.Font.Italic, "Keep undef features")
  
  Call rTarget.Delete
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_Merge()
  On Error GoTo PROPAGATE_ERROR
  
  Dim f1 As New InfoFormat
  Dim f2 As New InfoFormat
  Call Dev_ExpectTrue(f1.IsCompatibleWith(f2), "Empty to empty")
  On Error Resume Next
  Call f1.MergeWith(f2)
  Call Dev_ExpectNoError
  On Error GoTo PROPAGATE_ERROR
  
  Call f1.features_.Add(T_FF_FONT_SIZE, 42)
  Call Dev_ExpectTrue(f2.IsCompatibleWith(f1), "Empty to non-empty")
  Call Dev_ExpectTrue(f1.IsCompatibleWith(f2), "Non-empty to empty")
  
  Call Dev_NewCase("Complement features")
  Call f2.features_.Add(T_FF_BOLD, True)
  Call Dev_ExpectTrue(f2.IsCompatibleWith(f1))
  Call Dev_ExpectTrue(f1.IsCompatibleWith(f2))
  Call f1.MergeWith(f2)
  Call Dev_AssertEQ(2, f1.features_.Count)
  Call Dev_ExpectEQ(42, f1.features_(T_FF_FONT_SIZE), "Font size")
  Call Dev_ExpectEQ(True, f1.features_(T_FF_BOLD), "Bold")
  
  Call Dev_NewCase("Incompatible values")
  f2.features_.Item(T_FF_BOLD) = False
  Call Dev_ExpectFalse(f1.IsCompatibleWith(f2))
  Call Dev_ExpectFalse(f2.IsCompatibleWith(f1))
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

