VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "s_WordDocument"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

' TODO:
' Public Function GetAllHighlights() As Collection ' of ItemFragment
' Public Function GetBasicFragmentsCompound() As Collection ' of ItemFragment
' Public Function GetAllFragments() As Collection ' of ItemFragment

Private fso_ As Scripting.FileSystemObject
Private markup_ As MarkupAPI
Private word_ As API_WordWrapper
Private doc_ As InfoDocument

Public Function Setup()
  ' Mandatory setup function
  Set fso_ = New Scripting.FileSystemObject
  Call ResetTestFolder
  Set markup_ = New MarkupAPI
  Call markup_.LoadConfig(SetupTestConfig)
  Set word_ = New API_WordWrapper
  Call word_.SetApplication(ThisDocument.Application)
  Set doc_ = New InfoDocument
  Call doc_.Init(word_.NewDocument, markup_)
  Call doc_.SyncStyles
End Function

Public Function Teardown()
  ' Mandatory teardown function
  Call fso_.DeleteFolder(GetTestFolder)
  Call word_.ReleaseDocument
End Function

Public Function t_ApplyCategory()
  On Error GoTo PROPAGATE_ERROR
  ' =================12345678901234567
  doc_.doc_.Range = "Test1 test2 test3"
  
  Call Dev_NewCase("Valid category")
  Dim iTarget As Word.Range: Set iTarget = doc_.doc_.Range(7, 10)
  Dim iCat As InfoCategory: Set iCat = markup_.config_.categories_(1)
  Call Dev_ExpectTrue(doc_.ApplyCategory(iCat, iTarget))
  Call Dev_ExpectEQ(iCat.style_, iTarget.Style)
  Call Dev_ExpectNE(iCat.style_, doc_.doc_.Range(6, 7).Style)
  Call Dev_ExpectNE(iCat.style_, doc_.doc_.Range(10, 11).Style)
  
  Call Dev_NewCase("Incomplete category")
  Dim iNewCat As InfoCategory: Set iNewCat = markup_.config_.AddBasicCategory("Test1233")
  Call Dev_ExpectFalse(doc_.ApplyCategory(iNewCat, iTarget))
  Call Dev_ExpectEQ(iCat.style_, iTarget.Style)
  
  Call Dev_NewCase("Override category")
  Set iCat = markup_.config_.categories_(2)
  Call Dev_ExpectTrue(doc_.ApplyCategory(iCat, iTarget))
  Call Dev_ExpectEQ(iCat.style_, iTarget.Style)

  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_ApplyFragments()
  On Error GoTo PROPAGATE_ERROR
  ' =================12345678901234567
  doc_.doc_.Range = "Test1 test2 test3"
  
  Dim iTarget1 As Word.Range: Set iTarget1 = doc_.doc_.Range(7, 10)
  Dim iTarget2 As Word.Range: Set iTarget2 = doc_.doc_.Range(13, 17)
  Dim iCat1 As InfoCategory: Set iCat1 = markup_.config_.categories_(1)
  Dim iCat2 As InfoCategory: Set iCat2 = markup_.config_.categories_(2)
  
  Call Dev_NewCase("Empty input")
  Dim iFragments As New Collection
  On Error Resume Next
  Call Dev_ExpectTrue(doc_.ApplyFragments(iFragments))
  Call Dev_ExpectNoError
  On Error GoTo PROPAGATE_ERROR
  
  Call Dev_NewCase("Valid input")
  Dim iFrag As New ItemFragment
  Call iFrag.Init(iCat1.id_, iTarget1.Start, iTarget1.End)
  Call iFragments.Add(iFrag.Clone)
  Call iFrag.Init(iCat2.id_, iTarget2.Start, iTarget2.End)
  Call iFragments.Add(iFrag.Clone)
  Call Dev_ExpectTrue(doc_.ApplyFragments(iFragments))
  Call Dev_ExpectEQ(iCat1.style_, iTarget1.Style)
  Call Dev_ExpectEQ(iCat2.style_, iTarget2.Style)
  
  Call Dev_NewCase("Invalid category")
  iFragments.Item(1).id_ = 1337
  Call Dev_ExpectFalse(doc_.ApplyFragments(iFragments))

  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_AddCategory()
  On Error GoTo PROPAGATE_ERROR
  ' =================12345678901234567
  doc_.doc_.Range = "Test1 test2 test3"
  
  Call Dev_NewCase("Add to empty")
  Dim iTarget As Word.Range: Set iTarget = doc_.doc_.Range(7, 10)
  Dim iCat1 As InfoCategory: Set iCat1 = markup_.config_.categories_(1)
  Call Dev_ExpectTrue(doc_.AddCategory(iCat1, iTarget))
  Call Dev_ExpectEQ(iCat1.style_, iTarget.Style)
  Call Dev_ExpectNE(iCat1.style_, doc_.doc_.Range(6, 7).Style)
  Call Dev_ExpectNE(iCat1.style_, doc_.doc_.Range(10, 11).Style)
  
  Call Dev_NewCase("Incomplete category")
  Dim iNewCat As InfoCategory: Set iNewCat = markup_.config_.AddBasicCategory("Test1233")
  Call Dev_ExpectFalse(doc_.ApplyCategory(iNewCat, iTarget))
  Call Dev_ExpectEQ(iCat1.style_, iTarget.Style)
  
  Call Dev_NewCase("Add to same")
  Call Dev_ExpectTrue(doc_.AddCategory(iCat1, iTarget))
  Call Dev_ExpectEQ(iCat1.style_, iTarget.Style)
  
  Call Dev_NewCase("Add with new merge")
  Dim iCat2 As InfoCategory: Set iCat2 = markup_.config_.categories_(2)
  Dim nCount&: nCount = markup_.config_.categories_.Count
  Call Dev_ExpectFalse(doc_.AddCategory(iCat2, iTarget))

  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_SetCategoryVisible()
  On Error GoTo PROPAGATE_ERROR
  ' =================12345678901234567
  doc_.doc_.Range = "Test1 test2 test3"
  Dim iTarget As Word.Range: Set iTarget = doc_.doc_.Range(7, 10)
  Dim iCat As InfoCategory: Set iCat = markup_.config_.categories_(1)
  
  Call Dev_NewCase("No formatting")
  On Error Resume Next
  Call doc_.SetCategoryVisible(1337, bShow:=False)
  Call Dev_ExpectNoError("Invalid ID")
  On Error Resume Next
  Call doc_.SetCategoryVisible(1, bShow:=False)
  Call Dev_ExpectNoError("No formatting")
  On Error GoTo PROPAGATE_ERROR
  
  Call Dev_NewCase("Valid visibility")
  Call Dev_AssertTrue(doc_.ApplyCategory(iCat, iTarget))
  Call doc_.SetCategoryVisible(iCat.id_, bShow:=True)
  Call Dev_ExpectFalse(iTarget.Font.Hidden, "Show visible")
  Call doc_.SetCategoryVisible(iCat.id_, bShow:=False)
  Call Dev_ExpectTrue(iTarget.Font.Hidden, "Hide visible")
  Call doc_.SetCategoryVisible(iCat.id_, bShow:=False)
  Call Dev_ExpectTrue(iTarget.Font.Hidden, "Hide hiden")
  Call doc_.SetCategoryVisible(iCat.id_, bShow:=True)
  Call Dev_ExpectFalse(iTarget.Font.Hidden, "Show visible")
  
  Call Dev_NewCase("Multiple targets")
  Dim iTarget2 As Word.Range: Set iTarget2 = doc_.doc_.Range(1, 5)
  Call Dev_AssertTrue(doc_.ApplyCategory(iCat, iTarget2))
  Call doc_.SetCategoryVisible(iCat.id_, bShow:=False)
  Call Dev_ExpectTrue(iTarget.Font.Hidden)
  Call Dev_ExpectTrue(iTarget2.Font.Hidden)
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_SetCategoryHighlight()
  On Error GoTo PROPAGATE_ERROR
  ' =================12345678901234567
  doc_.doc_.Range = "Test1 test2 test3"
  Dim iTarget As Word.Range: Set iTarget = doc_.doc_.Range(7, 10)
  Dim iCat As InfoCategory: Set iCat = markup_.config_.categories_(1)
  
  Call Dev_NewCase("No formatting")
  On Error Resume Next
  Call doc_.SetCategoryHighlight(1337, bHighlight:=False)
  Call Dev_ExpectNoError("Invalid ID")
  On Error Resume Next
  Call doc_.SetCategoryHighlight(1, bHighlight:=False)
  Call Dev_ExpectNoError("No formatting")
  On Error GoTo PROPAGATE_ERROR
  
  Call Dev_NewCase("Valid visibility")
  Call Dev_AssertTrue(doc_.ApplyCategory(iCat, iTarget))
  Call doc_.SetCategoryHighlight(iCat.id_, bHighlight:=False)
  Call Dev_ExpectEQ(wdNoHighlight, iTarget.HighlightColorIndex, "Unhighlight plain")
  Call doc_.SetCategoryHighlight(iCat.id_, bHighlight:=True)
  Call Dev_ExpectEQ(wdYellow, iTarget.HighlightColorIndex, "Highlight plain")
  Call doc_.SetCategoryHighlight(iCat.id_, bHighlight:=True)
  Call Dev_ExpectEQ(wdYellow, iTarget.HighlightColorIndex, "Highlight highlighted")
  Call doc_.SetCategoryHighlight(iCat.id_, bHighlight:=False)
  Call Dev_ExpectEQ(wdNoHighlight, iTarget.HighlightColorIndex, "Unhighlight highlighted")
  
  Call Dev_NewCase("Multiple targets")
  Dim iTarget2 As Word.Range: Set iTarget2 = doc_.doc_.Range(1, 5)
  Call Dev_AssertTrue(doc_.ApplyCategory(iCat, iTarget2))
  Call doc_.SetCategoryHighlight(iCat.id_, bHighlight:=True)
  Call Dev_ExpectEQ(wdYellow, iTarget.HighlightColorIndex)
  Call Dev_ExpectEQ(wdYellow, iTarget.HighlightColorIndex)
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_IteratorFormat()
  On Error GoTo PROPAGATE_ERROR
  ' =================12345678901234567
  doc_.doc_.Range = "Test1 test2 test3"
  Dim iTarget As Word.Range: Set iTarget = doc_.doc_.Range(7, 10)
  Dim iCat As InfoCategory: Set iCat = markup_.config_.categories_(1)
  Dim iter As IteratorFormat
  
  Call Dev_NewCase("Invalid ID")
  On Error Resume Next
  Call Dev_ExpectNothing(doc_.GetFormatIterator(1337))
  Call Dev_ExpectNoError
  
  Call Dev_NewCase("Past end")
  On Error Resume Next
  Call Dev_ExpectNothing(doc_.GetFormatIterator(iCat.id_, doc_.doc_.Range.End + 1))
  Call Dev_ExpectNoError
  
  Call Dev_NewCase("No formatting")
  On Error Resume Next
  Set iter = doc_.GetFormatIterator(iCat.id_)
  Call Dev_ExpectNotNothing(iter)
  Call Dev_ExpectNoError
  Call Dev_ExpectEQ(doc_.doc_.Range(0, 0), iter.Range)
  Call Dev_ExpectNothing(iter.MoveNext)
  Call Dev_ExpectNothing(iter.MovePrev)
  On Error GoTo PROPAGATE_ERROR
  
  Call Dev_NewCase("Valid category iter")
  Call doc_.ApplyCategory(iCat, iTarget)
  Set iter = doc_.GetFormatIterator(iCat.id_)
  Call Dev_ExpectNotNothing(iter)
  Call Dev_ExpectEQ(doc_.doc_.Range(0, 0), iter.Range)
  Call Dev_ExpectEQ(iTarget, iter.MoveNext(), "Move next")
  Call Dev_ExpectEQ(iTarget, iter.Range)
  Call Dev_ExpectNothing(iter.MoveNext(), "Move next")
  Call Dev_ExpectEQ(iTarget, iter.Range)
  Call Dev_ExpectNothing(iter.MovePrev(), "Move prev")
  Call Dev_ExpectEQ(iTarget, iter.Range)
  
  Call Dev_NewCase("Valid format")
  Dim iTarget2 As Word.Range: Set iTarget2 = doc_.doc_.Range(13, 17)
  Call iCat.GetFormat(doc_.doc_).ApplyToWordFont(iTarget2.Font)
  Call Dev_ExpectEQ(iTarget2, iter.MoveNext(), "Move next")
  Call Dev_ExpectEQ(iTarget, iter.MovePrev(), "Move prev")
  
  Call Dev_NewCase("Mixed format")
  Dim iTarget3 As Word.Range: Set iTarget3 = doc_.doc_.Range(3, iTarget.Start)
  Call iCat.GetFormat(doc_.doc_).ApplyToWordFont(iTarget3.Font)
  Set iter = doc_.GetFormatIterator(iCat.id_)
  Call Dev_ExpectEQ(doc_.doc_.Range(iTarget3.Start, iTarget.End), iter.MoveNext())
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

Public Function t_IteratorStyle()
  On Error GoTo PROPAGATE_ERROR
  ' =================12345678901234567
  doc_.doc_.Range = "Test1 test2 test3"
  Dim iTarget As Word.Range: Set iTarget = doc_.doc_.Range(7, 10)
  Dim iCat As InfoCategory: Set iCat = markup_.config_.categories_(1)
  Dim iter As IteratorStyle
    
  Call Dev_NewCase("Invalid ID")
  On Error Resume Next
  Call Dev_ExpectNothing(doc_.GetStylesIterator(1337))
  Call Dev_ExpectNothing(doc_.GetPreciseIterator(1337))
  Call Dev_ExpectNoError
  
  Call Dev_NewCase("Past end")
  On Error Resume Next
  Call Dev_ExpectNothing(doc_.GetStylesIterator(iCat.id_, doc_.doc_.Range.End + 1))
  Call Dev_ExpectNothing(doc_.GetPreciseIterator(iCat.id_, doc_.doc_.Range.End + 1))
  Call Dev_ExpectNoError
  
  Call Dev_NewCase("No formatting")
  On Error Resume Next
  Set iter = doc_.GetStylesIterator(iCat.id_)
  Call Dev_ExpectNotNothing(iter)
  Call Dev_ExpectNoError
  Call Dev_ExpectEQ(doc_.doc_.Range(0, 0), iter.Range)
  Call Dev_ExpectNothing(iter.MoveNext)
  Call Dev_ExpectNothing(iter.MovePrev)

  On Error Resume Next
  Set iter = doc_.GetPreciseIterator(iCat.id_)
  Call Dev_ExpectNotNothing(iter)
  Call Dev_ExpectNoError
  Call Dev_ExpectEQ(doc_.doc_.Range(0, 0), iter.Range)
  Call Dev_ExpectNothing(iter.MoveNext)
  Call Dev_ExpectNothing(iter.MovePrev)
  On Error GoTo PROPAGATE_ERROR
  
  Call Dev_NewCase("Valid category iter")
  Call doc_.ApplyCategory(iCat, iTarget)
  Set iter = doc_.GetPreciseIterator(iCat.id_)
  Call Dev_ExpectNotNothing(iter)
  Call Dev_ExpectEQ(doc_.doc_.Range(0, 0), iter.Range)
  Call Dev_ExpectEQ(iTarget, iter.MoveNext(), "Move next")
  Call Dev_ExpectEQ(iTarget, iter.Range)
  Call Dev_ExpectNothing(iter.MoveNext(), "Move next")
  Call Dev_ExpectEQ(iTarget, iter.Range)
  Call Dev_ExpectNothing(iter.MovePrev(), "Move prev")
  Call Dev_ExpectEQ(iTarget, iter.Range)
  
  Call Dev_NewCase("Valid format")
  Dim iTarget2 As Word.Range: Set iTarget2 = doc_.doc_.Range(13, 17)
  Call iCat.GetFormat(doc_.doc_).ApplyToWordFont(iTarget2.Font)
  Call Dev_ExpectNothing(iter.MoveNext(), "Move next")
  Call Dev_ExpectNothing(iter.MovePrev(), "Move prev")
  
  Call Dev_NewCase("Multiple styles")
  Dim iTarget3 As Word.Range: Set iTarget3 = doc_.doc_.Range(3, iTarget.Start)
  Dim iTarget4 As Word.Range: Set iTarget4 = doc_.doc_.Range(iTarget.End, 12)
  Dim iCat2 As InfoCategory: Set iCat2 = markup_.config_.GetAt(2)
  Call doc_.ApplyCategory(iCat, iTarget3)
  Call doc_.ApplyCategory(iCat2, iTarget4)
  Set iter = New IteratorStyle: Call iter.Init(doc_.doc_, CColl(iCat.style_, iCat2.style_))
  Call Dev_ExpectEQ(doc_.doc_.Range(iTarget3.Start, iTarget4.End), iter.MoveNext())
  
  Exit Function
PROPAGATE_ERROR:
  Call Dev_LogError(Err.Number, Err.Description)
End Function

